/***************************************************************************
	file:		uart.c
	product:
	project:	8051 uart interface
	date: 		September.2013 - first release

	version 	1.0
	compiler:	Keil Cx51 V9.05

	mc:			AT89C5131A
	crystal:	12MHz, 24MHz

	author: 	Viktor Schabelski info@approbe.de
	licence:	Creative Commons Lizenz
                http://www.stack2Learn.de/cc.html

**************************************************/

/*-----------------------------------------------------------------------------
	includes
---------------------------------------------------*/
#include <at89c5131.h>
#include "uart.h"


/*-----------------------------------------------------------------------------
 *  global/constant variables	                                                *
 *----------------------------------------------------------------------------*/
#define UART_CARRIADGE_RETURN		((unsigned char) 0x0D)
#define UART_NEW_LINE						((unsigned char) 0x0A)


/*-----------------------------------------------------------------------------
	api
---------------------------------------------------*/
/*=======================================================================/**
*	@brief		Initialisierung der seriellen Schnittstelle
*						Mode 1, 9600Baud, 8 datenbit, 1 stopp bit, asynchron
*						fuer 12MHz oder 24MHz System Quarz
*           Timer 2
*	@param		systemCrysal: System Quarz in MHz; moegliche Werte 12 oder 24.
*           12 - 12 MHz; 24 - 24MHz
---------------------------------------------------*/
void Uart_Init(crystal_t systemCrysal)
{
    // init uart
    SCON |= 0x50;				// SM1 (2^6) = 1; REN (2^4) = 1;

//-------------------------------------------------
    // Timer2 init
    T2CON |= 0x30;		// TCLK (2^4) = 1; RCLK (2^5) = 1;
    TR2 = 1;					// Timmer2 run T2CON 2^2

    if(systemCrysal == _12MHz)
    {
        // Reloadwert fuer 12MHz Quarz
        RCAP2H = 0xFF;
        RCAP2L = 0xD9;
    }
    else if  (systemCrysal == _24MHz)
    {
        // Reloadwert fuer 24MHz Quarz
        RCAP2H = 0xFF;
        RCAP2L = 0xB2;
    }
    else {}

//-------------------------------------------------
    TI = 1;				// SCON; Ermoeglicht das Benutzen von printf()
}


/*=======================================================================*//**
*	@brief		Sende Character ueber UART
*	@param		uchDataByte: character
*	@return		UART_TRANSMISSION_FAIL
*	@return		UART_SUCCESS
---------------------------------------------------*/
errorCodeUart_t Uart_SendChar(unsigned char character)
{
    unsigned int maxWait = 1000;
    SBUF = character;
    while(TI == 0)
    {
        maxWait--;
        if (!maxWait)
        {
            TI = 0;
            return UART_TRANSMISSION_FAIL;
        }
    }
    TI = 0;
    return UART_SUCCESS;
}

/*=======================================================================/**
*	@brief		Sende String ueber Uart
*	@param		pString: string
*	@return		UART_TRANSMISSION_FAIL
*	@return		UART_BUSY
*	@return		UART_SUCCESS
---------------------------------------------------*/
errorCodeUart_t Uart_SendString(unsigned char* pString)
{
    errorCodeUart_t error = UART_BUSY;

    error = Uart_SendChar(UART_NEW_LINE);
    if (error != UART_SUCCESS)
    {
        return UART_TRANSMISSION_FAIL;
    }
    error = Uart_SendChar(UART_CARRIADGE_RETURN);
    if (error != UART_SUCCESS)
    {
        return UART_TRANSMISSION_FAIL;
    }
    while(*pString != 0)
    {
        error = Uart_SendChar(*pString);
        if (error != UART_SUCCESS)
        {
            return UART_TRANSMISSION_FAIL;
        }
        pString++;
    }
    return error;
}
